/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is gBrain.
 *
 * The Initial Developer of the Original Code is
 * Andreas Gohr.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const STATE_START = Components.interfaces.nsIWebProgressListener.STATE_START;
const STATE_STOP = Components.interfaces.nsIWebProgressListener.STATE_STOP;
const STATE_IS_DOCUMENT = Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT;

// we attach our own listener to this for watching pageloads
const gbrainWebProgress = Components.classes["@mozilla.org/docloaderservice;1"]
                                  .getService(Components.interfaces.nsIWebProgress)

var gbrain = {
    /**
     * Initialize the whole thing
     */
    onLoad: function() {
        // initialization code
        gbrain.initialized = true;
        gbrain.strings = document.getElementById("gbrain-strings");

        // Access the preferences
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);
        gbrain.prefs = prefs.getBranch("extensions.gbrain.");

        // Make the PasswordManager available
        gbrain.passwordManagerInternal = Components.classes["@mozilla.org/passwordmanager;1"]
                                         .createInstance(Components.interfaces.nsIPasswordManagerInternal);

        // make the button reflect the current state
        gbrain.updateToolbarButton();

        // add the listener
        gbrainWebProgress.addProgressListener(this,
            Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
    },

    /**
     * Clean up again
     */
    onUnLoad: function() {
        gbrainWebProgress.removeProgressListener(this);
    },

    /* -----------------------------------------------------------------
     * the following implements the nsIWebProgressListener Interface as
     * described at http://kb.mozillazine.org/Progress_listeners
     */

    QueryInterface: function(aIID) {
        if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
            aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
            aIID.equals(Components.interfaces.nsISupports))
            return this;
        throw Components.results.NS_NOINTERFACE;
    },

    // For definitions of the remaining functions see XulPlanet.com
    onLocationChange: function(aProgress, aRequest, aURI) { return 0; },
    onProgressChange: function() {return 0;},
    onStatusChange: function() {return 0;},
    onSecurityChange: function() {return 0;},
    onLinkIconAvailable: function() {return 0;},

    /**
     * This gets called when a page is loaded, we act on the stop state
     * and bookmark the page when the HTTP status is 200
     */
    onStateChange: function(aProgress, aRequest, aFlag, aStatus){
        // immedeatly return if disabled;
        if(!gbrain.prefs.getBoolPref("enabled")) return 0;
        if(!gbrain.initialized) return 0;
        if( (aFlag & STATE_STOP) ){
            gbrain.updateToolbarButton(); //make sure the button always is current

            try{
                aRequest.QueryInterface(Components.interfaces.nsIHttpChannel);
                if(aRequest.responseStatus == 200){
                    gbrain.bookmark(aProgress.DOMWindow);
                }
            }catch(e){
                // seems not to be an HTTP request - ignore and return
                return 0;
            }
        }
        return 0;
    },
   /* ----------------------------------------------------------------- */

    /**
     * Fetches the password from the PasswordManager
     */
    getPassword: function(){
        var host= {value:""}, username= {value:""}, password= {value:""};
        try{
            this.passwordManagerInternal.findPasswordEntry("gBrain.ext","gBrain","",host,username,password);
        }catch(e) {}
        return password.value;
    },

    /**
     * Print a message in an alert box
     */
    msg: function(msg){
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                    .getService(Components.interfaces.nsIPromptService);
        promptService.alert(window,"gBrain",msg);
    },

    /**
     * Handle middle click on the toolbar
     */
    onToolbarButtonClick: function(event) {
        if(event.button == 1){
            gbrain.openAndReuseOneTabPerURL('http://www.google.com/bookmarks/lookup?q=label:!gbrain');
        }
    },

    /**
     * Open an URL in the currently open Window
     */
    openInCurrentWindow: function(url){
        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                 .getService(Components.interfaces.nsIWindowMediator);
        var recentWindow = wm.getMostRecentWindow("navigator:browser");
        if (recentWindow) {
            // Use an existing browser window
            recentWindow.content.location.href=url;
        } else {
            // No browser windows are open, so open a new one.
            window.open(url);
        }
    },

    /**
     * Open an URL in a tab, reusing an existing one if URL is the same
     *
     * @link http://kb.mozillazine.org/Reusing_tabs_for_the_same_URL
     */
    openAndReuseOneTabPerURL: function(url) {
        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                 .getService(Components.interfaces.nsIWindowMediator);
        var browserEnumerator = wm.getEnumerator("navigator:browser");

        // Check each browser instance for our URL
        var found = false;
        while (browserEnumerator.hasMoreElements() && !found) {
            var browserInstance = browserEnumerator.getNext();
            browserInstance = browserInstance.getBrowser();

            // Check each tab of this browser instance
            var index = 0, numTabs = browserInstance.mPanelContainer.childNodes.length;
            while (index < numTabs && !found) {
                var currentTab = browserInstance.getBrowserAtIndex(index);
                if (url == currentTab.currentURI.spec) {
                    // The URL is already opened. Select its tab.
                    browserInstance.selectedTab = currentTab;
                    // Focus *this* browser
                    browserInstance.focus();
                    found = true;
                }
                index++;
            }
        }

        // Our URL isn't open. Open it now.
        if (!found) {
            var recentWindow = wm.getMostRecentWindow("navigator:browser");
            if (recentWindow) {
                // Use an existing browser window
                recentWindow.delayedOpenTab(url);
            } else {
                // No browser windows are open, so open a new one.
                window.open(url);
            }
        }
    },

    /**
     * Enable or disable the extension
     *
     * act can be
     * -1 for disable
     *  0 for toggle
     *  1 for enable
     */
    toggle: function(act){
        var set;
        if(act === 0){
            set = !gbrain.prefs.getBoolPref("enabled");
        }else if(act === 1){
            set = true;
        }else{
            set = false;
        }

        gbrain.prefs.setBoolPref("enabled",set);
        gbrain.updateToolbarButton();
    },

    /**
     * Set the toolbar style according to the enabled state
     */
    updateToolbarButton: function(){
        // reflect change on button
        var button = document.getElementById("gbrainbutton");
        if(!button) return;

        if(gbrain.prefs.getBoolPref("enabled")){
            button.className = 'toolbarbutton-1 gbrain-enabled';
            button.tooltipText = gbrain.strings.getString('tooltip_on');
        }else{
            button.className = 'toolbarbutton-1 gbrain-disabled';
            button.tooltipText = gbrain.strings.getString('tooltip_off');
        }
    },

    /**
     * Open the preferences window
     */
    openPrefs: function(){
        var features = "chrome,titlebar,toolbar,centerscreen,modal";
        window.openDialog('chrome://gbrain/content/gbrainSettings.xul', "Preferences", features);
        gbrain.updateToolbarButton();
    },

    /**
     * Prepares the bookmarking process
     *
     * @param win Object - The current window
     */
    bookmark: function(win){
        // get document info
        var url   = win.location.href;
        if(url.substr(0,4) != 'http') return; //we handle http only

        var ex  = gbrain.prefs.getComplexValue("exception",
                  Components.interfaces.nsISupportsString).data;
        try{
            var tmp  = ex.split("\n");
            var exar = new Array(0);
            for(var i=0; i<tmp.length; i++){
                if(!tmp[i].match(/^\s*$/)) exar[exar.length] = tmp[i];
            }
            if(exar.length){
                var exre = exar.join(')|(');
                exre = new RegExp('('+exre+')');
                if(exre.test(url)) return; // if re matches return
            }
        }catch(e){
            gbrain.msg(gbrain.strings.getString('error.recompile'));
            gbrain.toggle(-1);
            return;
        }

        var title = win.document.title;
        if(title == '') title = url.substr(0,60);

        // prepare the date labels
        var now = new Date();
        var y   = now.getFullYear();
        var m   = now.getMonth()+1;
        var d   = now.getDate();
        if(m < 10) m = '0'+m;
        if(d < 10) d = '0'+d;
        var dt  = 'y'+y+',m'+m+',d'+d;

        // get optional labels
        var lbls = gbrain.prefs.getComplexValue("labels",
                   Components.interfaces.nsISupportsString).data;
        if(lbls != '') lbls = ','+lbls;

        // prepare the bookmark
        var params = 'title='+encodeURIComponent(title);
        params += '&bkmk='+encodeURIComponent(url);
        params += '&labels='+encodeURIComponent('!gbrain,'+dt+lbls);

        // send the request
        gbrain.sendBookmark(params,true);
    },


    /**
     * Send an asynchronous bookmark request
     *
     * @param params    Array   - POST data containing the bookmark data
     * @param firsttry  Boolean - Is this the first try (second is done after login)
     */
    sendBookmark: function(params,firsttry){
    	
        // try to add the bookmark
        var req = new XMLHttpRequest();
        req.open('POST','http://www.google.com/bookmarks/mark',true);
	
	var http=Components.classes["@mozilla.org/network/protocol;1?name=http"].getService(Components.interfaces.nsIHttpProtocolHandler);
        var useragent=http.userAgent;
        
        req.setRequestHeader('Content-Type','application/x-www-form-urlencoded; charset=UTF-8');
	req.setRequestHeader('User-Agent', useragent+" GoogleToolbarFF");

        // prepare answer handling
        req.onreadystatechange = function () {
            if (req.readyState != 4) return;

            if(req.status != 200){
                // problem with google's server - disable gbrain
                gbrain.msg(gbrain.strings.getString('error.bkbadhttp'));
                gbrain.toggle(-1);
                return;
            }

            // we don't get a response on success
            if(!req.responseText.length) return;

            // still here and first try? probably not logged in
            if(firsttry){
                gbrain.sendLogin(params);
                return;
            }

            // woah, still here? something went wrong - we silently ignore it
        };

        // send the request
        req.send(params);
        
    },

    /**
     * Send an asynchronus login request, then continue bookmarking
     *
     * @param params    Array   - POST data containing the bookmark data
     */
    sendLogin: function(params){
        // fetch userdata from prefs
        var user  = gbrain.prefs.getComplexValue("username",
                    Components.interfaces.nsISupportsString).data;
//        var pass  = gbrain.prefs.getComplexValue("password",
//                    Components.interfaces.nsISupportsString).data;
        var pass = gbrain.getPassword();

        var creds = 'Email='+encodeURIComponent(user);
        creds += '&Passwd='+encodeURIComponent(pass);
        creds += '&nui=1&hl=en&rmShown=1&null=Sign+in&service=bookmarks';

        // prepare request and answer handling
        req = new XMLHttpRequest();
        req.open('POST','https://www.google.com/accounts/LoginAuth',true);
        req.setRequestHeader('Content-Type','application/x-www-form-urlencoded; charset=UTF-8');
        req.setRequestHeader('Referer','https://www.google.com/accounts/ServiceLogin');
        req.onreadystatechange = function () {
            if (req.readyState != 4) return;

            if(req.status != 200){
                gbrain.msg(gbrain.strings.getString('error.loginbadhttp'));
                gbrain.toggle(-1);
                return
            }

            // looks like login failed
            if(req.responseText.indexOf('class="errormsg"') != -1){
                gbrain.msg(gbrain.strings.getString('error.loginfail'));
                gbrain.toggle(-1);
                return;
            }

            // continue bookmarking
            gbrain.sendBookmark(params,false);
        };
        // send request
        req.send(creds);
    },

    /**
     * Opens a prompt to add an exception to the exception list
     */
    addException: function(){
        //get current URL
        var url = window.content.location.href;
        url = url.split('.').join('\\.');
        url = url.replace(/\?.*$/,'');
        url = url.replace(/#.*$/,'');

        // no need to add unsupported URLs to the exception list
        if(url.substr(0,4) != 'http') return;

        // show the expression for modification by the user
        var input = {value: url};
        var check = {value: false }; //not used
        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
        var result = prompts.prompt(window, gbrain.strings.getString('exprompt.title'),
                        gbrain.strings.getString('exprompt.label'), input, null, check);
        if(!result) return;

        // fetch existing exception
        var ex  = gbrain.prefs.getComplexValue("exception",
                  Components.interfaces.nsISupportsString).data;
        if(ex.substr(ex.length-1) != "\n") ex += "\n";
        ex += input.value;

        var newex = Components.classes["@mozilla.org/supports-string;1"]
                     .createInstance(Components.interfaces.nsISupportsString);
        newex.data = ex;
        gbrain.prefs.setComplexValue("exception",
                    Components.interfaces.nsISupportsString, newex);

        return;
    }
};
window.addEventListener("load", function(e) { gbrain.onLoad(e); }, false);
window.addEventListener("unload", function(e) { gbrain.onUnLoad(e); }, false);

